/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2006 by Alexander D. Alexeev
//   http://alex-boss.narod.ru
//   mailto:Alex.D.Alexeev@gmail.com
//
// You may do whatever you like with this file, I just don't care.
// If you find this code useful, credits would be nice!
// If you have any suggestions, welcome!
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __LOG2VIEW_H
#define __LOG2VIEW_H

#ifdef LOGGING_ON

namespace _Log2View_
{

	void Log2View(LPCTSTR format, ...);
	inline void Log2ViewRect(LPCTSTR szName, const RECT rc)
	{
		Log2View(TEXT("%s left=%d top=%d right=%d bottom=%d"), szName, rc.left, rc.top, rc.right, rc.bottom);
	}
	void ShowLastError();
	void LogLastError();
#ifdef __RPCDCE_H__
	void Log2ViewGuid(LPCTSTR szName, const GUID guid);
#endif // __RPCDCE_H__
}

#define LOG2VIEW						_Log2View_::Log2View
#define LOG2VIEW_RECT				_Log2View_::Log2ViewRect
#ifdef __RPCDCE_H__
	#define LOG2VIEW_GUID				_Log2View_::Log2ViewGuid
#endif // __RPCDCE_H__
#define LOG2VIEW_LASTERROR	_Log2View_::LogLastError
#define SHOW_LASTERROR			_Log2View_::ShowLastError
#define LOG2VIEW_RGB(cr)		LOG2VIEW(TEXT("COLORREF %s:(r=%03d,g=%03d,b=%03d)"), #cr, GetRValue(cr), GetGValue(cr), GetBValue(cr))
#else // _DEBUG

namespace _Log2View_
{

inline void Log2View(LPCTSTR format, ...) {};
inline void Log2ViewRect(LPCTSTR szName, const RECT rc) {};
#ifdef __RPCDCE_H__
inline void Log2ViewGuid(LPCTSTR szName, const GUID guid) {};
#endif //__RPCDCE_H__
inline void ShowLastError() {};
inline void LogLastError() {};

}

#define LOG2VIEW						1 ? (void)0 : _Log2View_::Log2View
#define LOG2VIEW_RECT				1 ? (void)0 : _Log2View_::Log2ViewRect
#ifdef __RPCDCE_H__
#define LOG2VIEW_GUID				1 ? (void)0 : _Log2View_::Log2ViewGuid
#endif //__RPCDCE_H__
#define LOG2VIEW_LASTERROR	1 ? (void)0 : _Log2View_::LogLastError
#define SHOW_LASTERROR			1 ? (void)0 : _Log2View_::ShowLastError
#define LOG2VIEW_RGB(cr)		LOG2VIEW(TEXT("COLORREF %s:(r=%03d,g=%03d,b=%03d)"), #cr, GetRValue(cr), GetGValue(cr), GetBValue(cr))
#endif // _DEBUG

#ifdef LOGGING_ON

namespace _Log2View_
{

inline void Log2View(LPCTSTR format, ...)
{
  HWND hwndLog = FindWindow(NULL, TEXT("LogView"));
  if (hwndLog)
  {
    TCHAR *buffer = new TCHAR[65530];

    va_list argptr;
    va_start(argptr, format);

#ifndef _INC_SHLWAPI
    if(format)
     wvsprintf (buffer, format, argptr);
    else
     buffer[0] = TEXT('\0');
#else
    if(format)
     wvnsprintf (buffer, 65530-2, format, argptr);
    else
     buffer[0] = TEXT('\0');
#endif // _INC_SHLWAPI
    va_end (argptr);

		lstrcat(buffer, TEXT("\r\n"));
    if (buffer[0])
    {
      COPYDATASTRUCT str;
#ifdef _UNICODE
      str.dwData = 1;
#else
      str.dwData = 0;
#endif
			str.cbData = (lstrlen(buffer)+1)*sizeof(TCHAR);
			str.lpData = (void *)buffer;
      ::SendMessage(hwndLog, WM_COPYDATA, NULL, (LPARAM)&str);
    }
    delete [] buffer;
  }
}

inline void GetLastErrorText(LPVOID &lpMsgBuf)
{
	FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			GetLastError(),
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(LPTSTR) &lpMsgBuf,
			0,
			NULL 
	);
}


inline void LogLastError()
{
	LPVOID lpMsgBuf = NULL;
	GetLastErrorText(lpMsgBuf);
	Log2View(TEXT("LastError: %s"), (LPCTSTR)lpMsgBuf);
	LocalFree( lpMsgBuf );
}

inline void ShowLastError()
{
	LPVOID lpMsgBuf = NULL;
	GetLastErrorText(lpMsgBuf);
	MessageBox( NULL, (LPCTSTR)lpMsgBuf, TEXT("Error"), MB_OK | MB_ICONINFORMATION );
	LocalFree( lpMsgBuf );
}

#ifdef __RPCDCE_H__
inline void Log2ViewGuid(LPCTSTR szName, const GUID guid)
{
#ifdef _UNICODE
	unsigned short *pString;
#else
	unsigned char *pString;
#endif //_UNICODE
	UuidToString((UUID *)&guid, &pString);
	LPOLESTR lpszProgID;
	ProgIDFromCLSID(guid, &lpszProgID);
	
#ifdef _UNICODE
	Log2View(TEXT("(%s)=%s (ProgID may be:\"%s\""), szName, pString, lpszProgID);
#else
	int length = ::WideCharToMultiByte(CP_ACP, 0, lpszProgID, -1, NULL, 0, NULL, NULL);
	char *pszBuffer = new char[length];
	::WideCharToMultiByte(CP_ACP, 0, lpszProgID, -1, pszBuffer, length, NULL, NULL);
	Log2View(TEXT("(%s)=%s (ProgID may be:\"%s\""), szName, pString, pszBuffer);
	delete [] pszBuffer;
#endif // _UNICODE
	CoTaskMemFree(lpszProgID);
	RpcStringFree(&pString);
}
#endif //__RPCDCE_H__

}

#endif //LOGGING_ON

#endif // __LOG2VIEW_H
