// TrayIcon.cpp: implementation of the CTrayIcon class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TrayIcon.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BOOL CTrayIcon::CreateTrayIcon(HWND hWnd, UINT uMsgCallBack, LPCTSTR szTooltip, HICON hIcon, UINT uID, LPCTSTR szInfoTitle, LPCTSTR szInfo)
{
	m_bEnabled = ( GetVersion() & 0xff ) >= 4;
	if (!m_bEnabled) return FALSE;

	//Make sure Notification window is valid
	m_bEnabled = hWnd && ::IsWindow(hWnd);
	if (!m_bEnabled) return FALSE;
	
	//Make sure we avoid conflict with other messages
	//Tray only supports tooltip text up to 64 characters
	// load up the NOTIFYICONDATA structure

	m_tnd.cbSize = sizeof(NOTIFYICONDATA);
	m_tnd.hWnd	 = hWnd;
	m_tnd.uID	 = uID;
	m_tnd.hIcon  = hIcon;
	m_tnd.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP | NIF_INFO;
	m_tnd.uCallbackMessage = uMsgCallBack;
	m_tnd.uTimeout = 15000;
	
	if (szInfoTitle)
		lstrcpyn(m_tnd.szInfoTitle, szInfoTitle, 64);
	if (szInfo)
		lstrcpyn(m_tnd.szInfo, szInfo, 256);
	m_tnd.dwInfoFlags = NIIF_INFO;

	lstrcpy (m_tnd.szTip, szTooltip);

	// Set the tray icon
	m_bEnabled = Shell_NotifyIcon(NIM_ADD, &m_tnd);
	return m_bEnabled;
}

LRESULT CTrayIcon::OnTrayNotification(UINT wParam, LONG lParam) 
{
	//Return quickly if its not for this tray icon
	if (wParam != m_tnd.uID)
		return 0L;

	HMENU hMenu, hSubMenu;
	extern HMODULE g_hModule;

	// Clicking with right button brings up a context menu
	if (LOWORD(lParam) == WM_RBUTTONUP)
	{	
		hMenu = ::LoadMenu(g_hModule, MAKEINTRESOURCE(m_tnd.uID));
		if (!hMenu) 
			return 0;
		hSubMenu = ::GetSubMenu(hMenu, 0);
		if (!hSubMenu) 
			return 0;

		// Make first menu item the default (bold font)
		::SetMenuDefaultItem(hSubMenu, 0, TRUE);

		//Display and track the popup menu
		POINT pos;
		::GetCursorPos(&pos);

		::SetForegroundWindow(m_tnd.hWnd);  
		::TrackPopupMenu(hSubMenu, 0, pos.x, pos.y, 0, m_tnd.hWnd, NULL);

		// BUGFIX: See "PRB: Menus for Notification Icons Don't Work Correctly"
		::PostMessage(m_tnd.hWnd, WM_NULL, 0, 0);

		::DestroyMenu(hMenu);
	} 
	else if (LOWORD(lParam) == WM_LBUTTONUP) 
	{
		hMenu = ::LoadMenu(g_hModule, MAKEINTRESOURCE(m_tnd.uID));
		if (!hMenu) 
			return 0;
		hSubMenu = ::GetSubMenu(hMenu, 0);
		if (!hSubMenu) 
			return 0;

		// double click received, the default action is to execute first menu item
		::SetForegroundWindow(m_tnd.hWnd);
		::SendMessage(m_tnd.hWnd, WM_COMMAND, GetMenuItemID(hSubMenu, 0), 0);

		::DestroyMenu(hMenu);
	}


	return 1;
}
